@@ -27,16 +27,15 @@ from . import core as ma
 from .core import (
     MaskedArray, MAError, add, array, asarray, concatenate, filled, count,
     getmask, getmaskarray, make_mask_descr, masked, masked_array, mask_or,
-    nomask, ones, sort, zeros, getdata, get_masked_subclass, dot,
-    mask_rowcols
+    nomask, ones, sort, zeros, getdata, get_masked_subclass, dot
     )
 
 import numpy as np
 from numpy import ndarray, array as nxarray
-from numpy.core.multiarray import normalize_axis_index
-from numpy.core.numeric import normalize_axis_tuple
-from numpy.lib.function_base import _ureduce
-from numpy.lib.index_tricks import AxisConcatenator
+from numpy.lib.array_utils import normalize_axis_index, normalize_axis_tuple
+from numpy.lib._function_base_impl import _ureduce
+from numpy.lib._index_tricks_impl import AxisConcatenator
+from numpy._core.numeric import normalize_axis_tuple
 
 
 def issequence(seq):
@@ -71,12 +70,12 @@ def count_masked(arr, axis=None):
 
     Examples
     --------
-    >>> import numpy.ma as ma
+    >>> import numpy as np
     >>> a = np.arange(9).reshape((3,3))
-    >>> a = ma.array(a)
-    >>> a[1, 0] = ma.masked
-    >>> a[1, 2] = ma.masked
-    >>> a[2, 1] = ma.masked
+    >>> a = np.ma.array(a)
+    >>> a[1, 0] = np.ma.masked
+    >>> a[1, 2] = np.ma.masked
+    >>> a[2, 1] = np.ma.masked
     >>> a
     masked_array(
       data=[[0, 1, 2],
@@ -86,14 +85,14 @@ def count_masked(arr, axis=None):
             [ True, False,  True],
             [False,  True, False]],
       fill_value=999999)
-    >>> ma.count_masked(a)
+    >>> np.ma.count_masked(a)
     3
 
     When the `axis` keyword is used an array is returned.
 
-    >>> ma.count_masked(a, axis=0)
+    >>> np.ma.count_masked(a, axis=0)
     array([1, 1, 1])
-    >>> ma.count_masked(a, axis=1)
+    >>> np.ma.count_masked(a, axis=1)
     array([0, 2, 1])
 
     """
@@ -124,10 +123,19 @@ def masked_all(shape, dtype=float):
     --------
     masked_all_like : Empty masked array modelled on an existing array.
 
+    Notes
+    -----
+    Unlike other masked array creation functions (e.g. `numpy.ma.zeros`,
+    `numpy.ma.ones`, `numpy.ma.full`), `masked_all` does not initialize the
+    values of the array, and may therefore be marginally faster. However,
+    the values stored in the newly allocated array are arbitrary. For
+    reproducible behavior, be sure to set each element of the array before
+    reading.
+
     Examples
     --------
-    >>> import numpy.ma as ma
-    >>> ma.masked_all((3, 3))
+    >>> import numpy as np
+    >>> np.ma.masked_all((3, 3))
     masked_array(
       data=[[--, --, --],
             [--, --, --],
@@ -140,10 +148,10 @@ def masked_all(shape, dtype=float):
 
     The `dtype` parameter defines the underlying data type.
 
-    >>> a = ma.masked_all((3, 3))
+    >>> a = np.ma.masked_all((3, 3))
     >>> a.dtype
     dtype('float64')
-    >>> a = ma.masked_all((3, 3), dtype=np.int32)
+    >>> a = np.ma.masked_all((3, 3), dtype=np.int32)
     >>> a.dtype
     dtype('int32')
 
@@ -179,27 +187,36 @@ def masked_all_like(arr):
     --------
     masked_all : Empty masked array with all elements masked.
 
+    Notes
+    -----
+    Unlike other masked array creation functions (e.g. `numpy.ma.zeros_like`,
+    `numpy.ma.ones_like`, `numpy.ma.full_like`), `masked_all_like` does not
+    initialize the values of the array, and may therefore be marginally
+    faster. However, the values stored in the newly allocated array are
+    arbitrary. For reproducible behavior, be sure to set each element of the
+    array before reading.
+
     Examples
     --------
-    >>> import numpy.ma as ma
+    >>> import numpy as np
     >>> arr = np.zeros((2, 3), dtype=np.float32)
     >>> arr
     array([[0., 0., 0.],
            [0., 0., 0.]], dtype=float32)
-    >>> ma.masked_all_like(arr)
+    >>> np.ma.masked_all_like(arr)
     masked_array(
       data=[[--, --, --],
             [--, --, --]],
       mask=[[ True,  True,  True],
             [ True,  True,  True]],
-      fill_value=1e+20,
+      fill_value=np.float64(1e+20),
       dtype=float32)
 
     The dtype of the masked array matches the dtype of `arr`.
 
     >>> arr.dtype
     dtype('float32')
-    >>> ma.masked_all_like(arr).dtype
+    >>> np.ma.masked_all_like(arr).dtype
     dtype('float32')
 
     """
@@ -251,10 +268,12 @@ class _fromnxfunction:
         npfunc = getattr(np, self.__name__, None)
         doc = getattr(npfunc, '__doc__', None)
         if doc:
-            sig = self.__name__ + ma.get_object_signature(npfunc)
+            sig = ma.get_object_signature(npfunc)
             doc = ma.doc_note(doc, "The function is applied to both the _data "
                                    "and the _mask, if any.")
-            return '\n\n'.join((sig, doc))
+            if sig:
+                sig = self.__name__ + sig + "\n\n"
+            return sig + doc
         return
 
     def __call__(self, *args, **params):
@@ -398,7 +417,7 @@ def apply_along_axis(func1d, axis, arr, *args, **kwargs):
         dtypes.append(np.asarray(res).dtype)
         outarr = zeros(outshape, object)
         outarr[tuple(ind)] = res
-        Ntot = np.product(outshape)
+        Ntot = np.prod(outshape)
         k = 1
         while k < Ntot:
             # increment the index
@@ -418,7 +437,7 @@ def apply_along_axis(func1d, axis, arr, *args, **kwargs):
         j = i.copy()
         j[axis] = ([slice(None, None)] * res.ndim)
         j.put(indlist, ind)
-        Ntot = np.product(outshape)
+        Ntot = np.prod(outshape)
         holdshape = outshape
         outshape = list(arr.shape)
         outshape[axis] = res.shape
@@ -483,6 +502,7 @@ if apply_over_axes.__doc__ is not None:
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.arange(24).reshape(2,3,4)
     >>> a[:,0,1] = np.ma.masked
     >>> a[:,1,:] = np.ma.masked
@@ -535,19 +555,27 @@ def average(a, axis=None, weights=None, returned=False, *,
     a : array_like
         Data to be averaged.
         Masked entries are not taken into account in the computation.
-    axis : int, optional
-        Axis along which to average `a`. If None, averaging is done over
-        the flattened array.
+    axis : None or int or tuple of ints, optional
+        Axis or axes along which to average `a`.  The default,
+        `axis=None`, will average over all of the elements of the input array.
+        If axis is a tuple of ints, averaging is performed on all of the axes
+        specified in the tuple instead of a single axis or all the axes as
+        before.
     weights : array_like, optional
-        The importance that each element has in the computation of the average.
-        The weights array can either be 1-D (in which case its length must be
-        the size of `a` along the given axis) or of the same shape as `a`.
-        If ``weights=None``, then all data in `a` are assumed to have a
-        weight equal to one.  The 1-D calculation is::
+        An array of weights associated with the values in `a`. Each value in
+        `a` contributes to the average according to its associated weight.
+        The array of weights must be the same shape as `a` if no axis is
+        specified, otherwise the weights must have dimensions and shape
+        consistent with `a` along the specified axis.
+        If `weights=None`, then all data in `a` are assumed to have a
+        weight equal to one.
+        The calculation is::
 
             avg = sum(a * weights) / sum(weights)
 
-        The only constraint on `weights` is that `sum(weights)` must not be 0.
+        where the sum is over all included elements.
+        The only constraint on the values of `weights` is that `sum(weights)`
+        must not be 0.
     returned : bool, optional
         Flag indicating whether a tuple ``(result, sum of weights)``
         should be returned as output (True), or just the result (False).
@@ -571,8 +599,20 @@ def average(a, axis=None, weights=None, returned=False, *,
         input data-type, otherwise. If returned, `sum_of_weights` is always
         `float64`.
 
+    Raises
+    ------
+    ZeroDivisionError
+        When all weights along axis are zero. See `numpy.ma.average` for a
+        version robust to this type of error.
+    TypeError
+        When `weights` does not have the same shape as `a`, and `axis=None`.
+    ValueError
+        When `weights` does not have dimensions and shape consistent with `a`
+        along specified `axis`.
+
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.array([1., 2., 3., 4.], mask=[False, False, True, True])
     >>> np.ma.average(a, weights=[3, 1, 0, 0])
     1.25
@@ -585,6 +625,22 @@ def average(a, axis=None, weights=None, returned=False, *,
             [4., 5.]],
       mask=False,
       fill_value=1e+20)
+    >>> data = np.arange(8).reshape((2, 2, 2))
+    >>> data
+    array([[[0, 1],
+            [2, 3]],
+           [[4, 5],
+            [6, 7]]])
+    >>> np.ma.average(data, axis=(0, 1), weights=[[1./4, 3./4], [1., 1./2]])
+    masked_array(data=[3.4, 4.4],
+             mask=[False, False],
+       fill_value=1e+20)
+    >>> np.ma.average(data, axis=0, weights=[[1./4, 3./4], [1., 1./2]])
+    Traceback (most recent call last):
+        ...
+    ValueError: Shape of weights must be consistent
+    with shape of a along specified axis.
+
     >>> avg, sumweights = np.ma.average(x, axis=0, weights=[1, 2, 3],
     ...                                 returned=True)
     >>> avg
@@ -605,7 +661,8 @@ def average(a, axis=None, weights=None, returned=False, *,
     a = asarray(a)
     m = getmask(a)
 
-    # inspired by 'average' in numpy/lib/function_base.py
+    if axis is not None:
+        axis = normalize_axis_tuple(axis, a.ndim, argname="axis")
 
     if keepdims is np._NoValue:
         # Don't pass on the keepdims argument if one wasn't given.
@@ -619,7 +676,7 @@ def average(a, axis=None, weights=None, returned=False, *,
     else:
         wgt = asarray(weights)
 
-        if issubclass(a.dtype.type, (np.integer, np.bool_)):
+        if issubclass(a.dtype.type, (np.integer, np.bool)):
             result_dtype = np.result_type(a.dtype, wgt.dtype, 'f8')
         else:
             result_dtype = np.result_type(a.dtype, wgt.dtype)
@@ -630,22 +687,21 @@ def average(a, axis=None, weights=None, returned=False, *,
                 raise TypeError(
                     "Axis must be specified when shapes of a and weights "
                     "differ.")
-            if wgt.ndim != 1:
-                raise TypeError(
-                    "1D weights expected when shapes of a and weights differ.")
-            if wgt.shape[0] != a.shape[axis]:
+            if wgt.shape != tuple(a.shape[ax] for ax in axis):
                 raise ValueError(
-                    "Length of weights not compatible with specified axis.")
+                    "Shape of weights must be consistent with "
+                    "shape of a along specified axis.")
 
             # setup wgt to broadcast along axis
-            wgt = np.broadcast_to(wgt, (a.ndim-1)*(1,) + wgt.shape, subok=True)
-            wgt = wgt.swapaxes(-1, axis)
+            wgt = wgt.transpose(np.argsort(axis))
+            wgt = wgt.reshape(tuple((s if ax in axis else 1)
+                                    for ax, s in enumerate(a.shape)))
 
         if m is not nomask:
             wgt = wgt*(~a.mask)
             wgt.mask |= a.mask
 
-        scl = wgt.sum(axis=axis, dtype=result_dtype)
+        scl = wgt.sum(axis=axis, dtype=result_dtype, **keepdims_kw)
         avg = np.multiply(a, wgt,
                           dtype=result_dtype).sum(axis, **keepdims_kw) / scl
 
@@ -710,6 +766,7 @@ def median(a, axis=None, out=None, overwrite_input=False, keepdims=False):
 
     Examples
     --------
+    >>> import numpy as np
     >>> x = np.ma.array(np.arange(8), mask=[0]*4 + [1]*4)
     >>> np.ma.median(x)
     1.5
@@ -732,12 +789,8 @@ def median(a, axis=None, out=None, overwrite_input=False, keepdims=False):
         else:
             return m
 
-    r, k = _ureduce(a, func=_median, axis=axis, out=out,
+    return _ureduce(a, func=_median, keepdims=keepdims, axis=axis, out=out,
                     overwrite_input=overwrite_input)
-    if keepdims:
-        return r.reshape(k)
-    else:
-        return r
 
 
 def _median(a, axis=None, out=None, overwrite_input=False):
@@ -778,7 +831,7 @@ def _median(a, axis=None, out=None, overwrite_input=False):
             s = mid.sum(out=out)
             if not odd:
                 s = np.true_divide(s, 2., casting='safe', out=out)
-            s = np.lib.utils._median_nancheck(asorted, s, axis)
+            s = np.lib._utils_impl._median_nancheck(asorted, s, axis)
         else:
             s = mid.mean(out=out)
 
@@ -818,7 +871,7 @@ def _median(a, axis=None, out=None, overwrite_input=False):
         s = np.ma.sum(low_high, axis=axis, out=out)
         np.true_divide(s.data, 2., casting='unsafe', out=s.data)
 
-        s = np.lib.utils._median_nancheck(asorted, s, axis)
+        s = np.lib._utils_impl._median_nancheck(asorted, s, axis)
     else:
         s = np.ma.mean(low_high, axis=axis, out=out)
 
@@ -845,6 +898,21 @@ def compress_nd(x, axis=None):
     -------
     compress_array : ndarray
         The compressed array.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> arr = [[1, 2], [3, 4]]
+    >>> mask = [[0, 1], [0, 0]]
+    >>> x = np.ma.array(arr, mask=mask)
+    >>> np.ma.compress_nd(x, axis=0)
+    array([[3, 4]])
+    >>> np.ma.compress_nd(x, axis=1)
+    array([[1],
+           [3]])
+    >>> np.ma.compress_nd(x)
+    array([[3]])
+
     """
     x = asarray(x)
     m = getmask(x)
@@ -895,6 +963,7 @@ def compress_rowcols(x, axis=None):
 
     Examples
     --------
+    >>> import numpy as np
     >>> x = np.ma.array(np.arange(9).reshape(3, 3), mask=[[1, 0, 0],
     ...                                                   [1, 0, 0],
     ...                                                   [0, 0, 0]])
@@ -930,10 +999,31 @@ def compress_rows(a):
     This is equivalent to ``np.ma.compress_rowcols(a, 0)``, see
     `compress_rowcols` for details.
 
+    Parameters
+    ----------
+    x : array_like, MaskedArray
+        The array to operate on. If not a MaskedArray instance (or if no array
+        elements are masked), `x` is interpreted as a MaskedArray with
+        `mask` set to `nomask`. Must be a 2D array.
+
+    Returns
+    -------
+    compressed_array : ndarray
+        The compressed array.
+
     See Also
     --------
     compress_rowcols
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> a = np.ma.array(np.arange(9).reshape(3, 3), mask=[[1, 0, 0],
+    ...                                                   [1, 0, 0],
+    ...                                                   [0, 0, 0]])
+    >>> np.ma.compress_rows(a)
+    array([[6, 7, 8]])
+
     """
     a = asarray(a)
     if a.ndim != 2:
@@ -948,10 +1038,33 @@ def compress_cols(a):
     This is equivalent to ``np.ma.compress_rowcols(a, 1)``, see
     `compress_rowcols` for details.
 
+    Parameters
+    ----------
+    x : array_like, MaskedArray
+        The array to operate on.  If not a MaskedArray instance (or if no array
+        elements are masked), `x` is interpreted as a MaskedArray with
+        `mask` set to `nomask`. Must be a 2D array.
+
+    Returns
+    -------
+    compressed_array : ndarray
+        The compressed array.
+
     See Also
     --------
     compress_rowcols
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> a = np.ma.array(np.arange(9).reshape(3, 3), mask=[[1, 0, 0],
+    ...                                                   [1, 0, 0],
+    ...                                                   [0, 0, 0]])
+    >>> np.ma.compress_cols(a)
+    array([[1, 2],
+           [4, 5],
+           [7, 8]])
+
     """
     a = asarray(a)
     if a.ndim != 2:
@@ -959,6 +1072,95 @@ def compress_cols(a):
     return compress_rowcols(a, 1)
 
 
+def mask_rowcols(a, axis=None):
+    """
+    Mask rows and/or columns of a 2D array that contain masked values.
+
+    Mask whole rows and/or columns of a 2D array that contain
+    masked values.  The masking behavior is selected using the
+    `axis` parameter.
+
+      - If `axis` is None, rows *and* columns are masked.
+      - If `axis` is 0, only rows are masked.
+      - If `axis` is 1 or -1, only columns are masked.
+
+    Parameters
+    ----------
+    a : array_like, MaskedArray
+        The array to mask.  If not a MaskedArray instance (or if no array
+        elements are masked), the result is a MaskedArray with `mask` set
+        to `nomask` (False). Must be a 2D array.
+    axis : int, optional
+        Axis along which to perform the operation. If None, applies to a
+        flattened version of the array.
+
+    Returns
+    -------
+    a : MaskedArray
+        A modified version of the input array, masked depending on the value
+        of the `axis` parameter.
+
+    Raises
+    ------
+    NotImplementedError
+        If input array `a` is not 2D.
+
+    See Also
+    --------
+    mask_rows : Mask rows of a 2D array that contain masked values.
+    mask_cols : Mask cols of a 2D array that contain masked values.
+    masked_where : Mask where a condition is met.
+
+    Notes
+    -----
+    The input array's mask is modified by this function.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> a = np.zeros((3, 3), dtype=int)
+    >>> a[1, 1] = 1
+    >>> a
+    array([[0, 0, 0],
+           [0, 1, 0],
+           [0, 0, 0]])
+    >>> a = np.ma.masked_equal(a, 1)
+    >>> a
+    masked_array(
+      data=[[0, 0, 0],
+            [0, --, 0],
+            [0, 0, 0]],
+      mask=[[False, False, False],
+            [False,  True, False],
+            [False, False, False]],
+      fill_value=1)
+    >>> np.ma.mask_rowcols(a)
+    masked_array(
+      data=[[0, --, 0],
+            [--, --, --],
+            [0, --, 0]],
+      mask=[[False,  True, False],
+            [ True,  True,  True],
+            [False,  True, False]],
+      fill_value=1)
+
+    """
+    a = array(a, subok=False)
+    if a.ndim != 2:
+        raise NotImplementedError("mask_rowcols works for 2D arrays only.")
+    m = getmask(a)
+    # Nothing is masked: return a
+    if m is nomask or not m.any():
+        return a
+    maskedval = m.nonzero()
+    a._mask = a._mask.copy()
+    if not axis:
+        a[np.unique(maskedval[0])] = masked
+    if axis in [None, 1, -1]:
+        a[:, np.unique(maskedval[1])] = masked
+    return a
+
+
 def mask_rows(a, axis=np._NoValue):
     """
     Mask rows of a 2D array that contain masked values.
@@ -972,14 +1174,14 @@ def mask_rows(a, axis=np._NoValue):
 
     Examples
     --------
-    >>> import numpy.ma as ma
+    >>> import numpy as np
     >>> a = np.zeros((3, 3), dtype=int)
     >>> a[1, 1] = 1
     >>> a
     array([[0, 0, 0],
            [0, 1, 0],
            [0, 0, 0]])
-    >>> a = ma.masked_equal(a, 1)
+    >>> a = np.ma.masked_equal(a, 1)
     >>> a
     masked_array(
       data=[[0, 0, 0],
@@ -990,7 +1192,7 @@ def mask_rows(a, axis=np._NoValue):
             [False, False, False]],
       fill_value=1)
 
-    >>> ma.mask_rows(a)
+    >>> np.ma.mask_rows(a)
     masked_array(
       data=[[0, 0, 0],
             [--, --, --],
@@ -1023,14 +1225,14 @@ def mask_cols(a, axis=np._NoValue):
 
     Examples
     --------
-    >>> import numpy.ma as ma
+    >>> import numpy as np
     >>> a = np.zeros((3, 3), dtype=int)
     >>> a[1, 1] = 1
     >>> a
     array([[0, 0, 0],
            [0, 1, 0],
            [0, 0, 0]])
-    >>> a = ma.masked_equal(a, 1)
+    >>> a = np.ma.masked_equal(a, 1)
     >>> a
     masked_array(
       data=[[0, 0, 0],
@@ -1040,7 +1242,7 @@ def mask_cols(a, axis=np._NoValue):
             [False,  True, False],
             [False, False, False]],
       fill_value=1)
-    >>> ma.mask_cols(a)
+    >>> np.ma.mask_cols(a)
     masked_array(
       data=[[0, --, 0],
             [0, --, 0],
@@ -1075,6 +1277,15 @@ def ediff1d(arr, to_end=None, to_begin=None):
     --------
     numpy.ediff1d : Equivalent function for ndarrays.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> arr = np.ma.array([1, 2, 4, 7, 0])
+    >>> np.ma.ediff1d(arr)
+    masked_array(data=[ 1,  2,  3, -7],
+                 mask=False,
+           fill_value=999999)
+
     """
     arr = ma.asanyarray(arr).flat
     ed = arr[1:] - arr[:-1]
@@ -1104,6 +1315,32 @@ def unique(ar1, return_index=False, return_inverse=False):
     --------
     numpy.unique : Equivalent function for ndarrays.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> a = [1, 2, 1000, 2, 3]
+    >>> mask = [0, 0, 1, 0, 0]
+    >>> masked_a = np.ma.masked_array(a, mask)
+    >>> masked_a
+    masked_array(data=[1, 2, --, 2, 3],
+                mask=[False, False,  True, False, False],
+        fill_value=999999)
+    >>> np.ma.unique(masked_a)
+    masked_array(data=[1, 2, 3, --],
+                mask=[False, False, False,  True],
+        fill_value=999999)
+    >>> np.ma.unique(masked_a, return_index=True)
+    (masked_array(data=[1, 2, 3, --],
+                mask=[False, False, False,  True],
+        fill_value=999999), array([0, 1, 4, 2]))
+    >>> np.ma.unique(masked_a, return_inverse=True)
+    (masked_array(data=[1, 2, 3, --],
+                mask=[False, False, False,  True],
+        fill_value=999999), array([0, 1, 3, 1, 2]))
+    >>> np.ma.unique(masked_a, return_index=True, return_inverse=True)
+    (masked_array(data=[1, 2, 3, --],
+                mask=[False, False, False,  True],
+        fill_value=999999), array([0, 1, 4, 2]), array([0, 1, 3, 1, 2]))
     """
     output = np.unique(ar1,
                        return_index=return_index,
@@ -1132,6 +1369,7 @@ def intersect1d(ar1, ar2, assume_unique=False):
 
     Examples
     --------
+    >>> import numpy as np
     >>> x = np.ma.array([1, 3, 3, 3], mask=[0, 0, 0, 1])
     >>> y = np.ma.array([3, 1, 1, 1], mask=[0, 0, 0, 1])
     >>> np.ma.intersect1d(x, y)
@@ -1159,12 +1397,22 @@ def setxor1d(ar1, ar2, assume_unique=False):
     --------
     numpy.setxor1d : Equivalent function for ndarrays.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> ar1 = np.ma.array([1, 2, 3, 2, 4])
+    >>> ar2 = np.ma.array([2, 3, 5, 7, 5])
+    >>> np.ma.setxor1d(ar1, ar2)
+    masked_array(data=[1, 4, 5, 7],
+                 mask=False,
+           fill_value=999999)
+
     """
     if not assume_unique:
         ar1 = unique(ar1)
         ar2 = unique(ar2)
 
-    aux = ma.concatenate((ar1, ar2))
+    aux = ma.concatenate((ar1, ar2), axis=None)
     if aux.size == 0:
         return aux
     aux.sort()
@@ -1194,6 +1442,16 @@ def in1d(ar1, ar2, assume_unique=False, invert=False):
     -----
     .. versionadded:: 1.4.0
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> ar1 = np.ma.array([0, 1, 2, 5, 0])
+    >>> ar2 = [0, 2]
+    >>> np.ma.in1d(ar1, ar2)
+    masked_array(data=[ True, False,  True, False,  True],
+                 mask=False,
+           fill_value=True)
+
     """
     if not assume_unique:
         ar1, rev_idx = unique(ar1, return_inverse=True)
@@ -1235,6 +1493,16 @@ def isin(element, test_elements, assume_unique=False, invert=False):
     -----
     .. versionadded:: 1.13.0
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> element = np.ma.array([1, 2, 3, 4, 5, 6])
+    >>> test_elements = [0, 2]
+    >>> np.ma.isin(element, test_elements)
+    masked_array(data=[False,  True, False, False, False, False],
+                 mask=False,
+           fill_value=True)
+
     """
     element = ma.asarray(element)
     return in1d(element, test_elements, assume_unique=assume_unique,
@@ -1251,6 +1519,16 @@ def union1d(ar1, ar2):
     --------
     numpy.union1d : Equivalent function for ndarrays.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> ar1 = np.ma.array([1, 2, 3, 4])
+    >>> ar2 = np.ma.array([3, 4, 5, 6])
+    >>> np.ma.union1d(ar1, ar2)
+    masked_array(data=[1, 2, 3, 4, 5, 6],
+             mask=False,
+       fill_value=999999)
+
     """
     return unique(ma.concatenate((ar1, ar2), axis=None))
 
@@ -1268,6 +1546,7 @@ def setdiff1d(ar1, ar2, assume_unique=False):
 
     Examples
     --------
+    >>> import numpy as np
     >>> x = np.ma.array([1, 2, 3, 4], mask=[0, 1, 0, 1])
     >>> np.ma.setdiff1d(x, [1, 2])
     masked_array(data=[3, --],
@@ -1311,7 +1590,14 @@ def _covhelper(x, y=None, rowvar=True, allow_masked=True):
         tup = (None, slice(None))
     #
     if y is None:
-        xnotmask = np.logical_not(xmask).astype(int)
+        # Check if we can guarantee that the integers in the (N - ddof)
+        # normalisation can be accurately represented with single-precision
+        # before computing the dot product.
+        if x.shape[0] > 2 ** 24 or x.shape[1] > 2 ** 24:
+            xnm_dtype = np.float64
+        else:
+            xnm_dtype = np.float32
+        xnotmask = np.logical_not(xmask).astype(xnm_dtype)
     else:
         y = array(y, copy=False, ndmin=2, dtype=float)
         ymask = ma.getmaskarray(y)
@@ -1326,7 +1612,16 @@ def _covhelper(x, y=None, rowvar=True, allow_masked=True):
                     x._sharedmask = False
                     y._sharedmask = False
         x = ma.concatenate((x, y), axis)
-        xnotmask = np.logical_not(np.concatenate((xmask, ymask), axis)).astype(int)
+        # Check if we can guarantee that the integers in the (N - ddof)
+        # normalisation can be accurately represented with single-precision
+        # before computing the dot product.
+        if x.shape[0] > 2 ** 24 or x.shape[1] > 2 ** 24:
+            xnm_dtype = np.float64
+        else:
+            xnm_dtype = np.float32
+        xnotmask = np.logical_not(np.concatenate((xmask, ymask), axis)).astype(
+            xnm_dtype
+        )
     x -= x.mean(axis=rowvar)[tup]
     return (x, xnotmask, rowvar)
 
@@ -1383,6 +1678,24 @@ def cov(x, y=None, rowvar=True, bias=False, allow_masked=True, ddof=None):
     --------
     numpy.cov
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> x = np.ma.array([[0, 1], [1, 1]], mask=[0, 1, 0, 1])
+    >>> y = np.ma.array([[1, 0], [0, 1]], mask=[0, 0, 1, 1])
+    >>> np.ma.cov(x, y)
+    masked_array(
+    data=[[--, --, --, --],
+          [--, --, --, --],
+          [--, --, --, --],
+          [--, --, --, --]],
+    mask=[[ True,  True,  True,  True],
+          [ True,  True,  True,  True],
+          [ True,  True,  True,  True],
+          [ True,  True,  True,  True]],
+    fill_value=1e+20,
+    dtype=float64)
+
     """
     # Check inputs
     if ddof is not None and ddof != int(ddof):
@@ -1396,11 +1709,17 @@ def cov(x, y=None, rowvar=True, bias=False, allow_masked=True, ddof=None):
 
     (x, xnotmask, rowvar) = _covhelper(x, y, rowvar, allow_masked)
     if not rowvar:
-        fact = np.dot(xnotmask.T, xnotmask) * 1. - ddof
-        result = (dot(x.T, x.conj(), strict=False) / fact).squeeze()
+        fact = np.dot(xnotmask.T, xnotmask) - ddof
+        mask = np.less_equal(fact, 0, dtype=bool)
+        with np.errstate(divide="ignore", invalid="ignore"):
+            data = np.dot(filled(x.T, 0), filled(x.conj(), 0)) / fact
+        result = ma.array(data, mask=mask).squeeze()
     else:
-        fact = np.dot(xnotmask, xnotmask.T) * 1. - ddof
-        result = (dot(x, x.T.conj(), strict=False) / fact).squeeze()
+        fact = np.dot(xnotmask, xnotmask.T) - ddof
+        mask = np.less_equal(fact, 0, dtype=bool)
+        with np.errstate(divide="ignore", invalid="ignore"):
+            data = np.dot(filled(x, 0), filled(x.T.conj(), 0)) / fact
+        result = ma.array(data, mask=mask).squeeze()
     return result
 
 
@@ -1451,44 +1770,34 @@ def corrcoef(x, y=None, rowvar=True, bias=np._NoValue, allow_masked=True,
     for backwards compatibility with previous versions of this function.  These
     arguments had no effect on the return values of the function and can be
     safely ignored in this and previous versions of numpy.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> x = np.ma.array([[0, 1], [1, 1]], mask=[0, 1, 0, 1])
+    >>> np.ma.corrcoef(x)
+    masked_array(
+      data=[[--, --],
+            [--, --]],
+      mask=[[ True,  True],
+            [ True,  True]],
+      fill_value=1e+20,
+      dtype=float64)
+
     """
     msg = 'bias and ddof have no effect and are deprecated'
     if bias is not np._NoValue or ddof is not np._NoValue:
         # 2015-03-15, 1.10
         warnings.warn(msg, DeprecationWarning, stacklevel=2)
-    # Get the data
-    (x, xnotmask, rowvar) = _covhelper(x, y, rowvar, allow_masked)
-    # Compute the covariance matrix
-    if not rowvar:
-        fact = np.dot(xnotmask.T, xnotmask) * 1.
-        c = (dot(x.T, x.conj(), strict=False) / fact).squeeze()
-    else:
-        fact = np.dot(xnotmask, xnotmask.T) * 1.
-        c = (dot(x, x.T.conj(), strict=False) / fact).squeeze()
-    # Check whether we have a scalar
+    # Estimate the covariance matrix.
+    corr = cov(x, y, rowvar, allow_masked=allow_masked)
+    # The non-masked version returns a masked value for a scalar.
     try:
-        diag = ma.diagonal(c)
+        std = ma.sqrt(ma.diagonal(corr))
     except ValueError:
-        return 1
-    #
-    if xnotmask.all():
-        _denom = ma.sqrt(ma.multiply.outer(diag, diag))
-    else:
-        _denom = diagflat(diag)
-        _denom._sharedmask = False  # We know return is always a copy
-        n = x.shape[1 - rowvar]
-        if rowvar:
-            for i in range(n - 1):
-                for j in range(i + 1, n):
-                    _x = mask_cols(vstack((x[i], x[j]))).var(axis=1)
-                    _denom[i, j] = _denom[j, i] = ma.sqrt(ma.multiply.reduce(_x))
-        else:
-            for i in range(n - 1):
-                for j in range(i + 1, n):
-                    _x = mask_cols(
-                            vstack((x[:, i], x[:, j]))).var(axis=1)
-                    _denom[i, j] = _denom[j, i] = ma.sqrt(ma.multiply.reduce(_x))
-    return c / _denom
+        return ma.MaskedConstant()
+    corr /= ma.multiply.outer(std, std)
+    return corr
 
 #####--------------------------------------------------------------------------
 #---- --- Concatenation helpers ---
@@ -1505,6 +1814,8 @@ class MAxisConcatenator(AxisConcatenator):
     mr_class
 
     """
+    __slots__ = ()
+
     concatenate = staticmethod(concatenate)
 
     @classmethod
@@ -1528,20 +1839,23 @@ class mr_class(MAxisConcatenator):
     """
     Translate slice objects to concatenation along the first axis.
 
-    This is the masked array version of `lib.index_tricks.RClass`.
+    This is the masked array version of `r_`.
 
     See Also
     --------
-    lib.index_tricks.RClass
+    r_
 
     Examples
     --------
+    >>> import numpy as np
     >>> np.ma.mr_[np.ma.array([1,2,3]), 0, 0, np.ma.array([4,5,6])]
     masked_array(data=[1, 2, 3, ..., 4, 5, 6],
                  mask=False,
            fill_value=999999)
 
     """
+    __slots__ = ()
+
     def __init__(self):
         MAxisConcatenator.__init__(self, 0)
 
@@ -1579,6 +1893,7 @@ def ndenumerate(a, compressed=True):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.arange(9).reshape((3, 3))
     >>> a[1, 0] = np.ma.masked
     >>> a[1, 2] = np.ma.masked
@@ -1648,6 +1963,7 @@ def flatnotmasked_edges(a):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.arange(10)
     >>> np.ma.flatnotmasked_edges(a)
     array([0, 9])
@@ -1705,6 +2021,7 @@ def notmasked_edges(a, axis=None):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.arange(9).reshape((3, 3))
     >>> m = np.zeros_like(a)
     >>> m[1:, 1:] = 1
@@ -1754,6 +2071,7 @@ def flatnotmasked_contiguous(a):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.arange(10)
     >>> np.ma.flatnotmasked_contiguous(a)
     [slice(0, 10, None)]
@@ -1815,6 +2133,7 @@ def notmasked_contiguous(a, axis=None):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.arange(12).reshape((3, 4))
     >>> mask = np.zeros_like(a)
     >>> mask[1:, :-1] = 1; mask[0, 1] = 1; mask[-1, 0] = 0
@@ -1844,7 +2163,7 @@ def notmasked_contiguous(a, axis=None):
     a = asarray(a)
     nd = a.ndim
     if nd > 2:
-        raise NotImplementedError("Currently limited to atmost 2D array.")
+        raise NotImplementedError("Currently limited to at most 2D array.")
     if axis is None or nd == 1:
         return flatnotmasked_contiguous(a)
     #
@@ -1916,6 +2235,7 @@ def clump_unmasked(a):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.masked_array(np.arange(10))
     >>> a[[0, 1, 2, 6, 8, 9]] = np.ma.masked
     >>> np.ma.clump_unmasked(a)
@@ -1955,6 +2275,7 @@ def clump_masked(a):
 
     Examples
     --------
+    >>> import numpy as np
     >>> a = np.ma.masked_array(np.arange(10))
     >>> a[[0, 1, 2, 6, 8, 9]] = np.ma.masked
     >>> np.ma.clump_masked(a)
